@@ -349,14 +349,86 @@ class Accuracy(Metric):
     return MeanStat.new(correct, 1.)
 
 
-def _target_weight(target: jnp.ndarray,
-                   masked_target_values: Tuple[int, ...]) -> jnp.ndarray:
+def get_target_weight(target: jnp.ndarray,
+                      masked_target_values: Tuple[int, ...]) -> jnp.ndarray:
   target_weight = jnp.ones_like(target, dtype=jnp.float32)
   for mv in masked_target_values:
     target_weight *= (target != mv)
   return target_weight
 
 
+@dataclasses.dataclass
+class TopKAccuracy(Metric):
+  """Metric for top k accuracy.
+
+  This metric computes the number of times where the correct class
+  is among the top k classes predicted.
+
+  Example: top 3 accuracy
+
+  - Dog => [Dog, Cat, Bird, Mouse, Penguin] ✓
+  - Cat => [Bird, Mouse, Cat, Penguin, Dog] ✓
+  - Dog => [Dog, Cat, Bird, Penguin, Mouse] ✓
+  - Bird => [Bird, Cat, Mouse, Penguin, Dog] ✓
+  - Cat => [Cat, Bird, Mouse, Dog, Penguin] ✓
+  - Cat => [Cat, Mouse, Dog, Penguin, Bird] ✓
+  - Mouse => [Penguin, Cat, Dog, Mouse, Bird] x
+  - Penguin => [Dog, Mouse, Cat, Penguin, Bird] x
+
+  6 correct predictions in top 3 predicted classes / 8 total examples
+  = .75 top 3 accuracy
+
+  Top k accuracy, also known as top n accuracy,
+  is a useful metric when it comes to recommendations.
+  One example would be the word recommendations on a virtual keyboard
+  where three suggested words are displayed.
+
+  For k=1, we strongly recommend using :class:`Accuracy` to avoid an
+  unnecessary argsort. k < 1 will return 0. and k >= num_classes will
+  return 1.
+
+  If two or more classes have the same prediction, the classes will be
+  considered in order of lowest to highest indices.
+
+  Example::
+
+    example = {'y': jnp.array(2)}
+    prediction = jnp.array([0, 0.5, 0.2])
+    metric = TopKAccuracy(k=2)
+    print(metric.evaluate_example(example, prediction))
+    # MeanStat(accum=1, weight=1) => 1
+
+  Attributes:
+    k: Number of top elements to look at for computing accuracy.
+    target_key: Key name in ``example`` for target.
+    pred_key: Key name in ``prediction`` for unnormalized model output pred.
+  """
+  k: int
+  target_key: str = 'y'
+  pred_key: Optional[str] = None
+
+  def zero(self) -> Stat:
+    return MeanStat.new(0., 0.)
+
+  def evaluate_example(self, example: SingleExample,
+                       prediction: SinglePrediction) -> Stat:
+
+    """Computes top k accuracy for a single example.
+
+    Args:
+      example: One example with target in range [0, num_classes) of shape [1].
+      prediction: Unnormalized prediction for ``example`` of shape [num_clases].
+
+    Returns:
+      MeanStat for top k accuracy for a single example.
+    """
+    target = example[self.target_key]
+    pred = prediction if self.pred_key is None else prediction[self.pred_key]
+    top_k_pred = jnp.argsort(-pred)[:self.k]
+    correct = jnp.any(top_k_pred == target).astype(jnp.float32)
+    return MeanStat.new(correct, 1.)
+
+
 @dataclasses.dataclass
 class SequenceTokenCrossEntropyLoss(Metric):
   """Metric for token cross entropy loss for a sequence example.
@@ -403,7 +475,7 @@ class SequenceTokenCrossEntropyLoss(Metric):
     """
     target = example[self.target_key]
     pred = prediction if self.pred_key is None else prediction[self.pred_key]
-    target_weight = _target_weight(target, self.masked_target_values)
+    target_weight = get_target_weight(target, self.masked_target_values)
     token_loss = unreduced_cross_entropy_loss(target, pred)
     if self.per_position:
       return MeanStat.new(token_loss * target_weight, target_weight)
@@ -449,7 +521,7 @@ class SequenceCrossEntropyLoss(Metric):
     """
     target = example[self.target_key]
     pred = prediction if self.pred_key is None else prediction[self.pred_key]
-    target_weight = _target_weight(target, self.masked_target_values)
+    target_weight = get_target_weight(target, self.masked_target_values)
     token_loss = unreduced_cross_entropy_loss(target, pred)
     # Change weight from number of non masked target tokens to 1 if the sequence
     # contains any non masked tokens or 0 if the entire sequence is masked.
@@ -514,7 +586,7 @@ class SequenceTokenAccuracy(Metric):
     if self.logits_mask is not None:
       logits_mask = jnp.array(self.logits_mask)
       pred += logits_mask
-    target_weight = _target_weight(target, self.masked_target_values)
+    target_weight = get_target_weight(target, self.masked_target_values)
     correct = (target == jnp.argmax(pred, axis=-1)).astype(jnp.float32)
     if self.per_position:
       return MeanStat.new(correct * target_weight, target_weight)
@@ -522,6 +594,79 @@ class SequenceTokenAccuracy(Metric):
         jnp.sum(correct * target_weight), jnp.sum(target_weight))
 
 
+@dataclasses.dataclass
+class SequenceTokenTopKAccuracy(Metric):
+  """Metric for token top k accuracy for a sequence example.
+
+  For more information on the top k accuracy metric,
+  refer to the :class:`TopKAccuracy` docstring.
+
+  Example::
+
+    example = {'y': jnp.array([1, 2, 2, 1, 3, 0])}
+    prediction = jnp.array([[0, 1, 0.5, 0], [1, 0.5, 0, 0], [0.8, 0, 0.7, 0],
+                            [0.5, 1, 0, 0], [0, 0.5, 0, 1], [0.5, 0, 0.9, 0]])
+    logits_mask = (0., 0., 0., jnp.NINF)
+    metric = SequenceTokenTopKAccuracy(k=2, logits_mask=logits_mask)
+    print(metric.evaluate_example(example, prediction))
+    # MeanStat(accum=3, weight=5) => 0.6
+
+    per_position_metric = SequenceTokenTopKAccuracy(k=2, logits_mask=logits_mask, per_position=True)
+    print(per_position_metric.evaluate_example(example, prediction))
+    # MeanStat(accum=[1., 0., 1., 1., 0., 0.], weight=[1., 1., 1., 1., 1., 0.]) => [1., 0., 1., 1., 0., 0.]
+
+  Attributes:
+    k: Number of top elements to look at for computing accuracy.
+    target_key: Key name in ``example`` for target.
+    pred_key: Key name in ``prediction`` for unnormalized model output pred.
+    masked_target_values: Target values that should be ignored in computation.
+      This is typically used to ignore padding values in computation.
+    logits_mask: Mask of shape [num_classes] to be applied for preds. This is
+      typically used to discount predictions for out-of-vocabulary tokens.
+    per_position: Whether to keep output statistic per position or sum across
+      positions for the entire sequence.
+  """
+  k: int
+  target_key: str = 'y'
+  pred_key: Optional[str] = None
+  masked_target_values: Tuple[int, ...] = (0,)
+  # logits_mask cannot be a jnp.ndarray nor np.ndarray because they are not
+  # hashable and `Metric`s must be hashable for `evaluate_model`.
+  logits_mask: Optional[Tuple[float, ...]] = None
+  per_position: bool = False
+
+  def zero(self) -> MeanStat:
+    return MeanStat.new(0., 0.)
+
+  def evaluate_example(self, example: SingleExample,
+                       prediction: SinglePrediction) -> MeanStat:
+    """Computes token top k accuracy for a single sequence example.
+
+    Args:
+      example: One example with target in range [0, num_classes) of shape
+        [max_length].
+      prediction: Unnormalized prediction for ``example`` of shape
+        [max_length, num_classes]
+
+    Returns:
+      MeanStat for token top k accuracy for a single sequence example or at each
+        token position if ``per_position`` is ``True``.
+    """
+    target = example[self.target_key]
+    pred = prediction if self.pred_key is None else prediction[self.pred_key]
+    if self.logits_mask is not None:
+      logits_mask = jnp.array(self.logits_mask)
+      pred += logits_mask
+    target_weight = get_target_weight(target, self.masked_target_values)
+    top_k_pred = jnp.argsort(-pred, axis=1)[:, :self.k]
+    correct = jnp.any(
+        jnp.transpose(top_k_pred) == target, axis=0).astype(jnp.float32)
+    if self.per_position:
+      return MeanStat.new(correct * target_weight, target_weight)
+    return MeanStat.new(
+        jnp.sum(correct * target_weight), jnp.sum(target_weight))
+
+
 @dataclasses.dataclass
 class SequenceTokenCount(Metric):
   """Metric for count of non masked tokens for a sequence example.
@@ -558,7 +703,7 @@ class SequenceTokenCount(Metric):
     """
     del prediction
     target = example[self.target_key]
-    target_weight = _target_weight(target, self.masked_target_values)
+    target_weight = get_target_weight(target, self.masked_target_values)
     return SumStat.new(jnp.sum(target_weight))
 
 
@@ -601,7 +746,7 @@ class SequenceCount(Metric):
     """
     del prediction
     target = example[self.target_key]
-    target_weight = _target_weight(target, self.masked_target_values)
+    target_weight = get_target_weight(target, self.masked_target_values)
     return SumStat.new(jnp.any(target_weight).astype(jnp.float32))
 
 
@@ -647,7 +792,7 @@ class SequenceTruncationRate(Metric):
     """
     del prediction
     target = example[self.target_key]
-    target_weight = _target_weight(target, self.masked_target_values)
+    target_weight = get_target_weight(target, self.masked_target_values)
     not_empty = jnp.sum(jnp.any(target_weight))
     target_is_truncated = jnp.all(target != self.eos_target_value)
     return MeanStat.new(target_is_truncated * not_empty, not_empty)
@@ -699,7 +844,7 @@ class SequenceTokenOOVRate(Metric):
     """
     del prediction
     target = example[self.target_key]
-    target_weight = _target_weight(target, self.masked_target_values)
+    target_weight = get_target_weight(target, self.masked_target_values)
     target_oov = jnp.ones_like(target, dtype=jnp.float32)
     for oov_value in self.oov_target_values:
       target_oov *= (target == oov_value)
@@ -745,7 +890,7 @@ class SequenceLength(Metric):
     """
     del prediction
     target = example[self.target_key]
-    target_weight = _target_weight(target, self.masked_target_values)
+    target_weight = get_target_weight(target, self.masked_target_values)
     return MeanStat.new(jnp.sum(target_weight), jnp.sum(jnp.any(target_weight)))
 
 
@@ -800,3 +945,79 @@ class PerDomainMetric(Metric):
     return jax.tree_util.tree_multimap(
         where, self.base.evaluate_example(example, prediction),
         self.base.zero())
+
+
+@dataclasses.dataclass
+class ConfusionMatrix(Metric):
+  """Metric for making a Confusion Matrix.
+
+  A confusion matrix is an nxn matrix often used to describe the performance
+  of a classification model on a set of test data for which the true values are
+  known. The model's predictions are represented through the columns, and the
+  known data values through the rows. This allows one to view in which areas the
+  model is doing well, as well as where there is room for improvement. For each
+  row in the confusion matrix, if there are a lot of numbers outside of the main
+  diagonal, the model is not doing so well in respect to when it is supposed to
+  output that row's relative output class.
+
+  Theoretical Example::
+
+                Predicted P     Predicted N
+
+    Actual P       TP               FN
+
+    Actual N       FP               TN
+
+    **This is for a binary classification model but the same concept applies
+    to any model with n outputs. Notice that the TPs and TNs will always
+    lie in the main diagonal of the matrix.
+
+  Example::
+
+    example = {'y': jnp.array(2)}
+    prediction = jnp.array([0., 1., 0.])
+    metric = ConfusionMatrix(num_classes=3)
+    print(metric.evaluate_example(example, prediction))
+    # SumStat(accum=DeviceArray([[0., 0., 0.],
+    #                            [0., 0., 0.],
+    #                            [0., 1., 0.]], dtype=float32)) => [[0. 0. 0.]
+    #                                                               [0. 0. 0.]
+    #                                                               [0. 1. 0.]]
+
+  Attributes:
+    target_key: Key name in ``example`` for target.
+    pred_key: Key name in ``prediction`` for unnormalized model output pred.
+    num_classes: Number of output classes of the model. Used to generate a
+      matrix of shape [num_classes, num_classes].
+  """
+  num_classes: int
+  target_key: str = 'y'
+  pred_key: Optional[str] = None
+
+  def zero(self) -> SumStat:
+    return SumStat.new(jnp.zeros((self.num_classes, self.num_classes)))
+
+  def evaluate_example(self, example: SingleExample,
+                       prediction: SinglePrediction) -> SumStat:
+    """Computes a Confusion Matrix for a single example.
+
+    Args:
+      example: One example with target in range [0, num_classes) of shape [1].
+      prediction: Unnormalized prediction for ``example`` of shape [num_classes]
+
+    Returns:
+      SumStat for the Confusion Matrix for a single example.
+
+    Raises:
+      ValueError: If the num_classes attribute is not equal to the number of
+        output classes of the model.
+    """
+    target = example[self.target_key]
+    pred = prediction if self.pred_key is None else prediction[self.pred_key]
+    if self.num_classes != len(pred):
+      raise ValueError('Make sure num_classes is equal to the number of output '
+                       f'classes of the model. num_classes: {self.num_classes} '
+                       f'number of output classes of the model: {len(pred)}')
+    pred_idx = jnp.argmax(pred, axis=-1)
+    confusion_matrix = jnp.zeros((self.num_classes, self.num_classes))
+    return SumStat.new(confusion_matrix.at[target, pred_idx].set(1))
