@@ -1,4 +1,4 @@
-# Copyright 2020 Google LLC
+# Copyright 2021 Google LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -13,28 +13,36 @@
 # limitations under the License.
 """Toy regression models."""
 
-import collections
+from fedjax.core import models
 
-from fedjax import core
 import haiku as hk
 import jax.numpy as jnp
 import numpy as np
 
 
-def create_regression_model() -> core.Model:
-  """Creates toy regression model."""
+def create_regression_model() -> models.Model:
+  """Creates toy regression model.
+
+  Matches the model used in:
+
+  Communication-Efficient Agnostic Federated Averaging
+    Jae Ro, Mingqing Chen, Rajiv Mathews, Mehryar Mohri, Ananda Theertha Suresh
+    https://arxiv.org/abs/2104.02748
+
+  Returns:
+    Model
+  """
 
   def forward_pass(batch):
     network = hk.Sequential([hk.Linear(1, with_bias=False)])
     return jnp.mean(network(batch['x']))
 
-  def loss(batch, preds):
-    return core.MeanMetric(
-        total=jnp.square(jnp.mean(batch['y']) - preds), count=1.0)
+  def train_loss(batch, preds):
+    return jnp.square(jnp.mean(batch['y']) - preds)
 
   transformed_forward_pass = hk.transform(forward_pass)
-  sample_batch = collections.OrderedDict(x=np.zeros((1, 1)), y=np.zeros((1,)))
-  return core.create_model_from_haiku(
+  sample_batch = {'x': np.zeros((1, 1)), 'y': np.zeros((1,))}
+  return models.create_model_from_haiku(
       transformed_forward_pass=transformed_forward_pass,
       sample_batch=sample_batch,
-      loss_fn=loss)
+      train_loss=train_loss)
