@@ -18,8 +18,9 @@ import os.path
 from typing import Any, Optional
 
 from absl import logging
+from fedjax.core import util
 
-import tensorflow as tf
+tf = util.import_tf()
 
 
 class Logger:
